package de.tuberlin.cs.ni.neubauer.mpcd.gain;

import java.util.List;
import java.util.Map;

import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.JoinPair;
import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.Pair;
import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.ProposedJoins;

public class DefaultGainTrackManager implements GainTrackManager
{
	GainTracker[] trackers;
	public Boolean verbose;
	
	public DefaultGainTrackManager(int partitions, Boolean verbose)
	{
		this.trackers = new GainTracker[partitions];
		this.verbose = verbose;
	}

	@Override
	public JoinPair nextJoin()
	{
		List<Pair> pairs = null;
		double bestGain = -2;
		int bestDimension = -1;
		
		for (int i=0; i<trackers.length; i++)
		{
			ProposedJoins prop = trackers[i].getBestJoins();
			if (prop==null)
			{
				continue;
			}
			if (prop.gain>bestGain)
			{
				bestGain = prop.gain;
				pairs = prop.pairs;
				bestDimension = i;
			}
		}
		
		if (pairs==null)
		{
			return null;
		}
		
		Pair pair;
		if (pairs.size()>1)
			pair = tieBreak(pairs);
		else
			pair = pairs.get(0);
		
		return new JoinPair(pair, bestDimension, bestGain);
	}

	@Override
	public void setPairs(int domain, Map<Pair, Double> pairValues)
	{
		this.trackers[domain].setPairs(pairValues);
	}

	@Override
	public void setPair(int domain, Pair pair, double value)
	{
		this.trackers[domain].setPair(pair, value);
	}

	@Override
	/**
	 * just get one pair if several are available
	 */
	public Pair tieBreak(List<Pair> pairs)
	{
		return pairs.get(0);
	}

	@Override
	public GainTracker get(int domain)
	{
		return this.trackers[domain];
	}

	@Override
	public void put(int domain, GainTracker gainTracker)
	{
		this.trackers[domain] = gainTracker;
	}

	@Override
	public Boolean verboseOutput() 
	{
		return this.verbose;
	}

	@Override
	public void updatePair(int domain, Pair pair, double delta)
	{
		this.trackers[domain].updatePair(pair, delta);
		
	}
}
