package de.tuberlin.cs.ni.neubauer.mpcd.gain;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.Pair;
import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.ProposedJoins;

/**
 * a slow but robust implementation of a gaintracker
 * @author nn
 *
 */
public class DefaultGainTracker implements GainTracker
{
	Map<Integer, Map<Integer, Double>> map;
	
	public DefaultGainTracker(Map<Integer, Map<Integer, Double>> map)
	{
		this.map = map;
	}

	@Override
	public ProposedJoins getBestJoins()
	{
		if (this.map.size()==0)
			return null;
		if (this.map.size()==1)
		{
			for (Map<Integer, Double> singleValue: this.map.values())
				if (singleValue.size()==0)
					return null;
		}
		
		ArrayList<Pair> pairs = new ArrayList<Pair>();
		double gain = -1;
		
		for (Map.Entry<Integer, Map<Integer, Double>> e: this.map.entrySet())
		{
			int smaller = e.getKey();
			Map<Integer, Double> biggerToGain = e.getValue(); 
			for (Map.Entry<Integer, Double> e2: biggerToGain.entrySet())
			{
				double newGain = e2.getValue();
				if (newGain<gain)
					continue;
				if (newGain>gain)
				{
					gain = newGain;
					pairs = new ArrayList<Pair>();
				}
				int bigger = e2.getKey();
				pairs.add(new Pair(smaller, bigger));
			}
		}
		
		return new ProposedJoins(pairs, gain);
	}


	@Override
	public void removeElement(int element)
	{
		// remove element from outer map
		map.remove(element);

		// remove element from inner maps
		for (Map.Entry<Integer, Map<Integer, Double>> e: this.map.entrySet())
			// save a bit because smaller element is first key so 
			// no need to look up if second key < element			
			if (e.getKey()<element)  
				e.getValue().remove(element);
	}

	@Override
	public void setPair(Pair pair, double value)
	{
		Map<Integer, Double> innerMap = null;
		if (!this.map.containsKey(pair.smaller_element))
		{
			innerMap = new HashMap<Integer, Double>();
			this.map.put(pair.smaller_element, innerMap);
		}
		else
		{
			innerMap = this.map.get(pair.smaller_element);
		}
		innerMap.put(pair.bigger_element, value);
	}

	@Override
	public void setPairs(Map<Pair, Double> pairValues)
	{
		for(Map.Entry<Pair, Double> e: pairValues.entrySet())
			setPair(e.getKey(), e.getValue());
	}

	@Override
	public double getPairGain(Pair pair)
	{
		if (!this.map.containsKey(pair.smaller_element))
			return -2;
		Map<Integer, Double> inner = this.map.get(pair.smaller_element);
		if (!inner.containsKey(pair.bigger_element))
			return -2;
		return inner.get(pair.bigger_element);
	}

	@Override
	public void updatePair(Pair pair, double delta)
	{
		Map<Integer, Double> innerMap = this.map.get(pair.smaller_element);
		double current = innerMap.get(pair.bigger_element);
		innerMap.put(pair.bigger_element, current+delta);
	}

	
	
}
