package de.tuberlin.cs.ni.neubauer.mpcd.io;


import java.io.File;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

import de.tuberlin.cs.ni.neubauer.mpcd.assignments.CommunityAssignment;
import de.tuberlin.cs.ni.neubauer.mpcd.assignments.CommunityAssignments;

public class FileExporter implements Exporter 
{
	String filename;
	Boolean outputSummary;
	
	public FileExporter(String filename)
	{
		this(filename, true);
	}
	
	public FileExporter(String filename, Boolean outputSummary)
	{
		this.outputSummary = outputSummary;
		this.filename = filename;
	}
	
	@Override
	public void writeResults(CommunityAssignments assignments, DataContainer data) throws IOException 
	{
		for (int dimension=0; dimension<assignments.size(); dimension++)
		{
			File file = new File(String.format("%s.%d", this.filename, dimension));
			Writer output = new BufferedWriter(new FileWriter(file));
			CommunityAssignment assignment = assignments.get(dimension);
			
			Map<Integer, Integer> byNode = data.translateMap(assignment.byNode(), dimension);
			Set<Integer> nodes = byNode.keySet();
			List<Integer> sortedNodes = new ArrayList<Integer>();
			sortedNodes.addAll(nodes);
			Collections.sort(sortedNodes);
			for (int node: sortedNodes)
			{
				output.write(String.format("%d: %d\n", node, data.getNewToOld(dimension, byNode.get(node))));
			}
			output.close();
		}
		
		if (this.outputSummary)
		{
			File file = new File(String.format("%s.summary.txt", this.filename));
			Writer output = new BufferedWriter(new FileWriter(file));
			output.write(String.format("step of optimal modularity: %d\n", assignments.getStep()));
			output.write(String.format("optimal modularity: %f", assignments.getValue()));
			output.close();
		}
		
	}

}
