package de.tuberlin.cs.ni.neubauer.mpcd.listeners;

import java.util.Set;

import de.tuberlin.cs.ni.neubauer.mpcd.Optimizer;
import de.tuberlin.cs.ni.neubauer.mpcd.assignments.CommunityAssignment;
import de.tuberlin.cs.ni.neubauer.mpcd.gain.GainTracker;
import de.tuberlin.cs.ni.neubauer.mpcd.modularity.MurataModularityState;
import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.JoinPair;
import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.Pair;

public class GainCheckListener extends NoShutDownListener
{

	/*
	 * this found two errors (9.12.): (testcases!)
	 *   - only affected/affected pairs
	 *   - took to-be-computed pairs from assignments still containing old communities
	 * @see de.tuberlin.cs.ni.neubauer.mpcd.listeners.OptimizationStepListener#onStep(de.tuberlin.cs.ni.neubauer.mpcd.Optimizer, de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.JoinPair)
	 */
	
	@Override
	public Boolean onStep(Optimizer optimizer, JoinPair joinPair) throws ListenerException
	{
		MurataModularityState modularityState = (MurataModularityState)optimizer.modularity;
		final double epsilon = 0.000001;
		
		for (int dimension=0; dimension<modularityState.getCommunityAssignments().assignments.length; dimension++)
		{
			GainTracker gt = 
				optimizer.gtm.get(dimension);
			CommunityAssignment communityAssignment = 
				modularityState.getCommunityAssignments().assignments[dimension];
			Set<Integer> communities = 
				communityAssignment.byCommunity().keySet();
			
			for (int smaller_community: communities)
				for (int bigger_community: communities)
				{
					if (smaller_community >= bigger_community)
						continue;
					Pair newPair = 
						new Pair(smaller_community, bigger_community);
					JoinPair newJoinPair = 
						new JoinPair(newPair, dimension);
					double realGain = 
						modularityState.computeGain(newJoinPair, false);
					double predictedGain = 
						gt.getPairGain(newPair);

					if (Math.abs(realGain-predictedGain) > epsilon)
					{
						String s = String.format("Wrong gain prediction for %s!\nright: %1.10f\nwrong: %1.10f",
								newJoinPair,
								realGain,
								predictedGain);
						throw (new ListenerException(s));
					}
				}
		}
		System.out.println("gain tables intact");
		return false;
	}


}
