package de.tuberlin.cs.ni.neubauer.mpcd.listeners;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;

import de.tuberlin.cs.ni.neubauer.mpcd.Optimizer;
import de.tuberlin.cs.ni.neubauer.mpcd.modularity.MurataModularityState;
import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.JoinPair;

public class UpdateCountListener implements OptimizationStepListener
{	
	ArrayList<Writer> output;
	static String newline = System.getProperty("line.separator");
	int numDimensions;
	
	public UpdateCountListener(String filename, int dimensions) throws IOException
	{
		this.numDimensions = dimensions;
		if (filename.length()>0)
		{
			this.output = new ArrayList<Writer>();
			for (int i=0; i<dimensions; i++)
			{
				File file = new File(String.format("%s.%d", filename, i));
				this.output.add(new BufferedWriter(new FileWriter(file)));
			}
		}
	}

	@Override
	public Boolean onStep(Optimizer optimizer, JoinPair joinPair) throws IOException
	{
		
		MurataModularityState mms = (MurataModularityState)optimizer.modularity;
		
		for (int i=0; i<this.numDimensions; i++)
		{
			int ratio = 0;
			if (mms.pairsComputed[i]>0)
				ratio = 100*mms.pairsChanged[i]/mms.pairsComputed[i];
			System.out.printf("  dim %d: %d pairs max, %d pairs updated, %d actually changed (%d%%)\n",
				i,
				mms.pairsMax[i],
				mms.pairsComputed[i],
				mms.pairsChanged[i],
				ratio);
		
			if (this.output!=null)
			{
				String report = String.format("%d %d %d %d%s",
						joinPair.dimension,
						mms.pairsMax[i],
						mms.pairsComputed[i],
						mms.pairsChanged[i],
						newline);
				this.output.get(i).write(report);
			}
		}	
		return false;
	}

	@Override
	public void shutDown()
	{
		if (this.output==null)
			return;
		for (Writer writer: this.output)
			try
			{
				writer.close();
			}
			catch (Exception e)
			{
				System.err.println("error closing UpdateCountListener's writer: " + e.getMessage());
			}
	}
	

}
