package de.tuberlin.cs.ni.neubauer.mpcd.modularity;

import java.util.HashMap;
import java.util.Map;


import de.tuberlin.cs.ni.neubauer.mpcd.edges.EdgeContainer;
import de.tuberlin.cs.ni.neubauer.mpcd.edges.EdgeContainerFactory;
import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.Pair;

/**
 * all domain-specific state-objects 
 * @author nn
 *
 */
public class DomainState
{
	
	/** to-domain -> EdgeContainer **/
	public Map<Integer, EdgeContainer> edges;
	
	/** to-domain -> (from-community -> modularity) */
	public Map<Integer, Map<Integer, Double>> modularity;
	
	int numNodes = -1;
	
	int myDimension;

	EdgeContainerFactory edgeContainerFactory;
	
	public DomainState(int myDimension, int partitions, Boolean unweightedDU, EdgeContainerFactory edgeContainerFactory)
	{
		this.edgeContainerFactory = edgeContainerFactory;
		this.myDimension = myDimension;
		this.modularity = new HashMap<Integer, Map<Integer, Double>>();
		
		this.edges = new HashMap<Integer, EdgeContainer>();
		for (int j = 0; j<partitions; j++)
		{
			if (myDimension==j)
				continue;
			Boolean unweighted = false;
			if (unweightedDU)
				unweighted = (myDimension==0 && j==1) || (myDimension==1 && j==0);
			this.edges.put(j, getEdgeContainer(unweighted));
			this.modularity.put(j, new HashMap<Integer, Double>());
		}
	}
	
	/**
	 * override this for usage of specialized EdgeContainers
	 * @param unweighted
	 * @return
	 */
	public EdgeContainer getEdgeContainer(Boolean unweighted)
	{
		return this.edgeContainerFactory.getEdgeContainer(unweighted);
	}
	
	public void setNumNodes(int numNodes)
	{
		this.numNodes = numNodes;
	}
	
	public int getNumNodes()
	{
		return this.numNodes;
	}
	
	public void join(int dimension, Pair pair)
	{
		for (Map.Entry<Integer, EdgeContainer> entry: this.edges.entrySet())
		{
			int otherDimension = entry.getKey();
			EdgeContainer edges = entry.getValue();
			if (dimension == this.myDimension)
			{
				edges.merge_from(pair);
			}
			else if (dimension == otherDimension)
			{
				edges.merge_to(pair);
			}
		}
	}
	
	@Override
	public String toString()
	{
		StringBuilder sb = new StringBuilder("\n[DomainState dim " + this.myDimension + "\n");
		
		sb.append("** Edges **\n");
		for (Map.Entry<Integer, EdgeContainer> entry: this.edges.entrySet())
		{
			sb.append(String.format("toDim=%d:\n%s\n", entry.getKey(), entry.getValue()));
		}
		
		additionalToString(sb);
		
		sb.append("]");
		return sb.toString();
	}	
	
	
	public void additionalToString(StringBuilder sb)
	{
		
	}
	
}