package de.tuberlin.cs.ni.neubauer.mpcd.modularity;

import java.util.HashMap;
import java.util.Map;

import de.tuberlin.cs.ni.neubauer.mpcd.edges.EdgeContainerFactory;
import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.CommunityWithStrength;
import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.Pair;

public class MurataDomainState extends DomainState
{
	/** dimension -> fromCommunity->best community */
	public Map<Integer, Map<Integer, CommunityWithStrength>> correspondingCommunities;
	
	public MurataDomainState(int i, int partitions, Boolean unweightedDU, EdgeContainerFactory edgeContainerFactory)
	{
		super(i, partitions, unweightedDU, edgeContainerFactory);
		this.correspondingCommunities = new HashMap<Integer, Map<Integer, CommunityWithStrength>>();
		for (int j = 0; j<partitions; j++)
		{
			if (i==j)
				continue;
			this.correspondingCommunities.put(j, new HashMap<Integer, CommunityWithStrength>());
		}
	}

	
	@Override
	public void join(int dimension, Pair pair)
	{
		super.join(dimension, pair);
		if (dimension!=this.myDimension)
			return;
		for (Map<Integer, CommunityWithStrength> map: this.correspondingCommunities.values())
		{
			map.remove(pair.bigger_element);
		}
	}
	
	
	@Override
	public void additionalToString(StringBuilder sb)
	{
		sb.append("** Corresponding communities **\n");
		for (Map.Entry<Integer, Map<Integer, CommunityWithStrength>> entry: this.correspondingCommunities.entrySet())
		{
			sb.append(String.format("toDim=%d:\n%s\n", entry.getKey(), entry.getValue()));
		}		
	}
	
}
