package de.tuberlin.cs.ni.neubauer.mpcd.modularity;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import de.tuberlin.cs.ni.neubauer.mpcd.edges.EdgeContainer;
import de.tuberlin.cs.ni.neubauer.mpcd.edges.EdgeContainerFactory;
import de.tuberlin.cs.ni.neubauer.mpcd.gain.GainTrackManager;
import de.tuberlin.cs.ni.neubauer.mpcd.gain.GainTracker;
import de.tuberlin.cs.ni.neubauer.mpcd.gain.GainTrackerFactory;
import de.tuberlin.cs.ni.neubauer.mpcd.io.DataContainer;
import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.CommunityWithStrength;
import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.JoinPair;
import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.Pair;

public class OMMS2 extends OptimizedMurataModularityState
{
	public OMMS2(DataContainer data, 
			EdgeContainerFactory edgeFactory, 
			GainTrackerFactory gainTrackerFactory, 
			Boolean unweightedDU)
{
	super(data, edgeFactory, gainTrackerFactory, unweightedDU);
}


	@Override
	void updateGain(JoinPair joinPair, GainTrackManager gtm)
	{
		int dimension = joinPair.dimension;
		int biggerElement = joinPair.pair.bigger_element;
		int smallerElement = joinPair.pair.smaller_element;
		MurataDomainState myDomainState = (MurataDomainState)this.domainStates.get(dimension);		
		
		Set<Integer> myAffected = this.lastJoinOptMetadata.affectedCommunities;
		Map<Integer, Set<Integer>> otherAffected = this.lastJoinOptMetadata.otherAffectedCommunities;
		
		/** first step: compute affected and connected communities */
		// otherDimension -> set of nodes in other dimension connected to my affected communities
		Map<Integer, Set<Integer>> otherConnected = new HashMap<Integer, Set<Integer>>();
		// otherDimension -> set of nodes in my dimension connected to affected communities in other dimension
		Map<Integer, Set<Integer>> myConnected = new HashMap<Integer, Set<Integer>>();
		
		Map<Integer, Set<Pair>> semiAffected = new HashMap<Integer, Set<Pair>> ();
		

		Map<Integer, Double> newWeight = new HashMap<Integer, Double>();
		
		
		// compute connected communities
		for (int otherDimension: otherAffected.keySet())
		{
			
			
			HashSet<Pair> currentSemi = new HashSet<Pair>();
			semiAffected.put(otherDimension, currentSemi);
			EdgeContainer myEdgesToOther = myDomainState.edges.get(otherDimension); 
			EdgeContainer otherEdgesToMe = this.domainStates.get(otherDimension).edges.get(dimension);
			
			newWeight.put(otherDimension, myEdgesToOther.getSize(joinPair.pair.smaller_element) + myEdgesToOther.getSize(joinPair.pair.bigger_element));
			
			Map<Integer, CommunityWithStrength> corrCommOther = 
				((MurataDomainState)this.domainStates.get(otherDimension)).correspondingCommunities.get(dimension);	
			
			// test: statt myAffected nur joinPair! (chocho)
			Set<Integer> onlyPair = new HashSet<Integer>(2);
			onlyPair.add(joinPair.pair.smaller_element);
			onlyPair.add(joinPair.pair.bigger_element);
			
			
			Set<Integer> other= myEdgesToOther.getConnectedFromAny(onlyPair); // myAffected
			
			// these guys here are highly annoying.
			// any community in OWN that's connected to any community in OTHER that's connected to joint pair
			// gets AFFECTED status, even though they are ONLY relevant if they stand a chance of taking 
			// correspondence status from or losing it to the new joint community
			// maybe I should monitor those more closely to see if this really pays out
			// have to find the nice little test set which I used to see if the error pops up again if I do
			// (because that's where I first found the need)
			for (int otherCommunity: other)
			{
				// ok it seems this is only required if annoying guy and join-pair have identical number of 
				// nodes. if this is true, we may actually get rid of the final 10% slack
				int currentCorrComm = corrCommOther.get(otherCommunity).community;
				if (currentCorrComm==smallerElement || currentCorrComm==biggerElement)
					continue;
				
				double connSmaller = myEdgesToOther.getStrength(smallerElement, otherCommunity);
				double connBigger = myEdgesToOther.getStrength(biggerElement, otherCommunity);
				double connCorrComm = myEdgesToOther.getStrength(currentCorrComm, otherCommunity);
				
				// connCorrComm==connSmaller || connCorrComm==connBigger ||  
				if (connCorrComm==connSmaller+connBigger)
				{
					// keep track of the fact that currentCorrComm is "our"
					// corrComm of otherCommunity which is in dings
					currentSemi.add(new Pair(currentCorrComm, otherCommunity));
				}
			}
			other.removeAll(otherAffected.get(otherDimension));
			
			
			otherConnected.put(otherDimension, other);
			
			
			
			Set<Integer> my = otherEdgesToMe.getConnectedFromAny(otherAffected.get(otherDimension));
			
			// brauchen wir das hier? definitiv notwendig bislang nur die andere Richtung
			/*
			Map<Integer, CommunityWithStrength> corrCommMy = 
				((MurataDomainState)this.domainStates.get(dimension)).correspondingCommunities.get(otherDimension);		
			
			Set<Integer> otherAff = otherAffected.get(otherDimension);			
			for (int myCommunity: my)
			{
				otherAff.add(corrCommMy.get(myCommunity).community);
			}
			*/
			
			my.removeAll(myAffected);
			myConnected.put(otherDimension, my);

		}

		
		/** second step: update modularities and corresponding communities */ 
		// optimization: we're catching all affected communities right in computeGain, makes things easier here
		performUpdates(joinPair);
		
		
		/** third step: join edges */
		for (DomainState domainstate: this.domainStates.values())
			domainstate.join(joinPair.dimension, joinPair.pair);
		
		
		/** fourth step: finally update gain */
		gtm.get(dimension).removeElement(biggerElement);
		myAffected.remove(biggerElement);
		
		
		// initialize excessive bookkeeping
		this.pairsComputed = new int[this.numDimensions];
		this.pairsChanged = new int[this.numDimensions];
		this.pairsMax = new int[this.numDimensions];
		this.connectedPairsComputed = new int[this.numDimensions];
		this.connectedPairsChanged = new int[this.numDimensions];
		this.affectedPairsComputed = new int[this.numDimensions];
		this.affectedPairsChanged = new int[this.numDimensions];
		
		// update my dimension
		Set<Integer> myAll = this.assignments.get(dimension).byCommunity().keySet();
		GainTracker gt = gtm.get(dimension);
		updateGainAffectedPairs(dimension, myAll, myAffected, gt);
		updateGainConnectedPairList(dimension, myConnected, gt);
		
		
		
		updateGainSemiAffectedPairs(dimension, semiAffected, myConnected, myAll, gt, newWeight, joinPair.pair.smaller_element);
		
		// update other dimension
		for (Map.Entry<Integer, Set<Integer>> e: otherAffected.entrySet())
		{
			int otherDimension = e.getKey();
			Set<Integer> affected = e.getValue();
			Set<Integer> otherAll = this.assignments.get(otherDimension).byCommunity().keySet(); 
			gt = gtm.get(otherDimension);
			updateGainAffectedPairs(otherDimension, otherAll, affected, gt);
			updateGainConnectedPairs(otherDimension, otherConnected.get(otherDimension), gt);
		}
	}
	
	void updateGainSemiAffectedPairs(int dimension, Map<Integer, Set<Pair>> semiAffected, Map<Integer, Set<Integer>> connected, Set<Integer> all, GainTracker gt, Map<Integer, Double> newWeight, int newID)
	{
		MurataDomainState myDomainState = (MurataDomainState)this.domainStates.get(dimension);
		Map<Integer, Set<Integer>> toJoin = new HashMap<Integer, Set<Integer>>();
		for (Map.Entry<Integer, Set<Pair>> e: semiAffected.entrySet())
		{
			int otherDimension = e.getKey();
			double jointWeight = newWeight.get(otherDimension); // weight of edges of joint pair
			Set<Integer> currentConnected = connected.get(otherDimension);
			EdgeContainer myEdgesToOther = myDomainState.edges.get(otherDimension); 
			//EdgeContainer otherEdgesToMe = this.domainStates.get(otherDimension).edges.get(dimension);
			//double norm = myEdgesToOther.norm();
			//double normSquare = norm*norm;
			
			for (Pair p: e.getValue())
			{
				int myCC = p.smaller_element;
				double myWeight = myEdgesToOther.getSize(myCC);
				int otherSource = p.bigger_element; // so otherSource might switch to newly joint thingie if we get too large
				Boolean isConnected = currentConnected.contains(myCC);
				Set<Integer> toJoinTo = null;
				if (toJoin.containsKey(myCC))
				{
					toJoinTo = toJoin.get(myCC);
				}
				else
				{
					toJoinTo = new HashSet<Integer>();
					toJoin.put(myCC, toJoinTo);
				}
				
				for (int otherCommunity: all)
				{
					if (isConnected)
					{
						// if both are connected, they'll be taken care of elsewhere
						if (currentConnected.contains(otherCommunity))
							continue;
					}
					double otherWeight = myEdgesToOther.getSize(otherCommunity);
					double myJoinWeight = myWeight+otherWeight;
					if (myJoinWeight<=jointWeight)
						continue;
					if (myEdgesToOther.getStrength(otherCommunity, otherSource)>0)
						continue;
					
					// if we're here, a join makes us as fatter as the current join comm
					// since change is only due to weight change, equal weight doesn't change gain
					// also, we're not getting any new connections which might make us a favourite
					
					if (myCC<otherCommunity)
					{
						toJoinTo.add(otherCommunity);
					}
					else
					{
						Set<Integer> otherJoinTo = null;
						if (toJoin.containsKey(otherCommunity))
						{
							otherJoinTo = toJoin.get(otherCommunity);
						}
						else
						{
							otherJoinTo = new HashSet<Integer>();
							toJoin.put(otherCommunity, otherJoinTo);
						}
						otherJoinTo.add(myCC);
					}
					
					
					/*
					this.pairsComputed[dimension] += 1;
					double gainDelta = normSquare*otherEdgesToMe.getSize(otherSource)*(myWeight-jointWeight);
					if (gainDelta!=0)
						this.pairsChanged[dimension] += 1;
					gt.updatePair(pair, -gainDelta);
					*/
				}
			}
		}
		
		for (Map.Entry<Integer, Set<Integer>> e: toJoin.entrySet())
		{
			int smaller = e.getKey();
			for (int bigger: e.getValue())
			{
				Pair pair = new Pair(smaller, bigger);
				JoinPair joinPair = new JoinPair(pair, dimension);
				this.pairsComputed[dimension] += 1;
				double newGain = computeGain(joinPair, false);
				if (gt.getPairGain(pair)!=newGain)
				{
					this.pairsChanged[dimension] += 1;
					gt.setPair(pair, newGain);
				}
			}
		}
				
	}	
}
