package de.tuberlin.cs.ni.neubauer.mpcd.modularity;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.CommunityWithStrength;
import de.tuberlin.cs.ni.neubauer.mpcd.smalltypes.JoinPair;

/**
 * storage class for OptimizedMurataModularity
 * doesn't really need to be publicly visible, just trying to clean up sources
 * @author nn
 *
 */
public class OptimizedMurataModularityGainMetaData 
{
	// dim from -> dim to -> community -> modularity
	public Map<Integer, Map<Integer, Map<Integer, Double>>> newModularity;
	
	// dim from -> dim to -> community -> corresponding
	public Map<Integer, Map<Integer, Map<Integer, CommunityWithStrength>>> newCorrespondingCommunities;
	
	public Set<Integer> affectedCommunities;
	public Map<Integer, Set<Integer>> otherAffectedCommunities;
	
	public OptimizedMurataModularityGainMetaData(JoinPair pair, int numDimensions)
	{
		this.newCorrespondingCommunities = new HashMap<Integer, Map<Integer, Map<Integer, CommunityWithStrength>>>();
		this.newModularity = new HashMap<Integer, Map<Integer, Map<Integer, Double>>>();
		this.affectedCommunities = new HashSet<Integer>();
		this.affectedCommunities.add(pair.pair.smaller_element);
		this.affectedCommunities.add(pair.pair.bigger_element);
		
		this.otherAffectedCommunities = new HashMap<Integer, Set<Integer>>();
		for (int i=0; i<numDimensions; i++)
			if (i!=pair.dimension)
				this.otherAffectedCommunities.put(i, new HashSet<Integer>());
	}
	

	
	
	public void storeCorrespondingCommunities (int from, int to, int community, CommunityWithStrength correspondingCommunity)
	{
		Map<Integer, Map<Integer, CommunityWithStrength>> dimensionMap = null;
		if (!this.newCorrespondingCommunities.containsKey(from))
		{
			dimensionMap = new HashMap<Integer, Map<Integer, CommunityWithStrength>>();
			this.newCorrespondingCommunities.put(from, dimensionMap);
		}
		else
		{
			dimensionMap = this.newCorrespondingCommunities.get(from);
		}
		
		Map<Integer, CommunityWithStrength> innerMap = null;
		if (!dimensionMap.containsKey(to))
		{
			innerMap = new HashMap<Integer, CommunityWithStrength>();
			dimensionMap.put(to, innerMap);
		}
		else
		{
			innerMap = dimensionMap.get(to);
		}
		innerMap.put(community, correspondingCommunity);
	}
	
	public Map<Integer, CommunityWithStrength> getCorrespondingCommunities(int from, int to)
	{
		if (!this.newCorrespondingCommunities.containsKey(from))
			return new HashMap<Integer, CommunityWithStrength>();
		Map<Integer, Map<Integer, CommunityWithStrength>> map = this.newCorrespondingCommunities.get(from);
		if (!map.containsKey(to))
			return new HashMap<Integer, CommunityWithStrength>();
		return map.get(to);
	}
	
	public void storeNewModularity(int from, int to, int community, double modularity)
	{
		Map<Integer, Map<Integer, Double>> dimensionMap = null;
		if (!this.newModularity.containsKey(from))
		{
			dimensionMap = new HashMap<Integer, Map<Integer, Double>>();
			this.newModularity.put(from, dimensionMap);
		}
		else
		{
			dimensionMap = this.newModularity.get(from);
		}
		
		Map<Integer, Double> innerMap = null;
		if (!dimensionMap.containsKey(to))
		{
			innerMap = new HashMap<Integer, Double>();
			dimensionMap.put(to, innerMap);
		}
		else
		{
			innerMap = dimensionMap.get(to);
		}
		innerMap.put(community, modularity);			
	}
	
	public Map<Integer, Double> getModularity(int from, int to)
	{
		if (!this.newModularity.containsKey(from))
			return new HashMap<Integer, Double>();
		Map<Integer, Map<Integer, Double>> map = this.newModularity.get(from);
		if (!map.containsKey(to))
			return new HashMap<Integer, Double>();
		return map.get(to);
		
	}
	
	public void addAffectedCommunity(int community)
	{
		this.affectedCommunities.add(community);
	}
	
	public void addOtherAffectedCommunity(int otherCommunity, int dimension)
	{
		this.otherAffectedCommunities.get(dimension).add(otherCommunity);
	}
	
	public void addOtherAffectedCommunities(Collection<Integer> otherCommunities, int dimension)
	{	
		this.otherAffectedCommunities.get(dimension).addAll(otherCommunities);
	}
}
